﻿using SushiBarContracts.BindingModel;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModel;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using Microsoft.Extensions.Logging;
using SushiBarDataModels.Enums;
using System.Xml.Linq;
using SushiBarBusinessLogic.MailWorker;

namespace SushiBarBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IClientStorage clientStorage;
        private readonly AbstractMailWorker abstractMailWorker;
        static readonly object blocking = new object();

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage,
            IClientStorage clientStorage, AbstractMailWorker abstractMailWorker)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            this.clientStorage = clientStorage;
            this.abstractMailWorker = abstractMailWorker;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var elem = _orderStorage.GetElement(model);
            if (elem == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement found. Id:{Id}", elem.Id);
            return elem;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                return false;
            }
            model.Status = OrderStatus.Принят;
            var order = _orderStorage.Insert(model);
            if (order == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            var clientView = clientStorage.GetElement(new() { Id = order.ClientId });

            if (clientView != null)
                SendMail(clientView, order);
            return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            lock (blocking)
            {
                return ChangeStatus(model, OrderStatus.Выполняется);
            }
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }

        private bool ChangeStatus(OrderBindingModel model, OrderStatus orderStatus)
        {
            CheckModel(model, false);
            var order = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });

            if (order == null)
            {
                _logger.LogWarning("Change status operation failed. Order not found");
                return false;
            }
            if (order.Status + 1 != orderStatus)
            {
                _logger.LogWarning("Change status operation failed. Incorrect new status: {orderStatus}. Current status: {Status}",
                    orderStatus, order.Status);
                return false;
            }

            if (order.ImplementerId.HasValue)
                model.ImplementerId = order.ImplementerId;

            model.Status = orderStatus;

            if (model.Status == OrderStatus.Готов)
            {
                model.DateImplement = DateTime.Now;
            }
            else
            {
                model.DateImplement = order.DateImplement;
            }
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Change status operation failed");
                return false;
            }

            var client = clientStorage.GetElement(new ClientSearchModel { Id = order.ClientId });
            if (client != null)
                SendMail(client, order);
            return true;
        }

        public void SendMail(ClientViewModel clientView, OrderViewModel orderView)
        {
            if (clientView == null && orderView == null)
            {
                return;
            }

            MailSendInfoBindingModel mailSendInfoBindingModel;

            if (orderView.Status == OrderStatus.Принят)
            {
                mailSendInfoBindingModel = new MailSendInfoBindingModel
                {
                    MailAddress = clientView!.Email,
                    Subject = $"Заказ под номером {orderView.Id}",
                    Text = $"Ваш заказ под номером {orderView.Id} от {orderView.DateCreate} ценой в {orderView.Sum} тыщ долларов" +
                    $"был принят"
                };
            }
            else
            {
                mailSendInfoBindingModel = new MailSendInfoBindingModel
                {
                    MailAddress = clientView!.Email,
                    Subject = $"Заказ под номером {orderView.Id}",
                    Text = $"Ваш заказ под номером {orderView.Id} от {orderView.DateCreate} ценой в {orderView.Sum} тыщ долларов " +
                    $"поменял статус на {orderView.Status}"
                };
            }
            abstractMailWorker.MailSendAsync(mailSendInfoBindingModel);
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
                throw new ArgumentNullException(nameof(model));

            if (!withParams) return;

            if (model.SushiId <= 0)
                throw new ArgumentNullException("Неверный идентификатор суши", nameof(model.SushiId));

            if (model.Count <= 0)
                throw new ArgumentNullException("Количество суш в заказе не может быть меньше нуля или равно нулю", nameof(model.Count));

            if (model.Sum <= 0)
                throw new ArgumentNullException("Стоимость заказа не может быть меньше нуля", nameof(model.Sum));

            if (model.DateImplement.HasValue && model.DateImplement < model.DateCreate)
                throw new ArithmeticException("Заказ должен быть выдан позже, чем был создан");

            _logger.LogInformation("Sushi. SushiId:{SushiId}. Count:{ Count}. Sum:{ Sum}. Id: { Id}",
                model.SushiId, model.Count, model.Sum, model.Id);
        }
    }
}
